// EFileBrowser	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <list>
#include <storage/path.h>
#include <storage/directory.h>
#include <storage/volumes.h>
#include "foldertreeview.h"

using namespace os;
using namespace std;

class FolderTreeView::Private
{
public:
	Message*			m_pcDirChangeMsg;
	Path				m_cPath;	
	String				m_cSearchString;
	bool				m_bLocked;
};

FolderTreeView::FolderTreeView( const Rect & cFrame, const String& cPath, uint32 nResizeMask ): TreeView( cFrame, "_folder_treeview", TreeView::F_NO_HEADER, nResizeMask )
{
	m = new FolderTreeView::Private();
        
	m->m_cPath = cPath;
	m->m_pcDirChangeMsg = NULL;
	m->m_bLocked = false;

	InsertColumn( "folders", 1 );
	SetExpandedMessage( new Message( MSG_EXPAND ) );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

FolderTreeView::~FolderTreeView()
{
	delete( m );
}

void FolderTreeView::AttachedToWindow()
{        
	SetTarget( this );
	TreeView::AttachedToWindow();
	SetTarget( this );
	ReadRoot();
}

void FolderTreeView::AllAttached()
{        
	SetTarget( this );
}


void FolderTreeView::DetachedFromWindow()
{
	TreeView::DetachedFromWindow();
}

void FolderTreeView::HandleMessage( Message* pcMessage )
{
// printf( "msg\n" );
	switch ( pcMessage->GetCode() )
	{
		case MSG_EXPAND:
			{
				TreeViewNode* node = dynamic_cast< TreeViewNode* >( GetRow( GetFirstSelected() ) );
				if( node->IsExpanded() )
					ExpandItem( GetFirstSelected(), node );
				else
					CollapseItem( GetFirstSelected(), node );
			break;
			}
	}
}

void FolderTreeView :: ReadDirectory( const String& path )
{
	try
	{
		String cName;
		Directory dir( path );
		dir.Rewind();

//		Clear( path );

	int i = 0;

		while( dir.GetNextEntry( &cName ) == 1 )
		{
			TreeViewStringNode* node = new TreeViewStringNode();
			node->SetExpanded( true );
			node->AppendString( cName );
			if( i < 4 )
				node->SetIndent( 0 );
			else
				node->SetIndent( 2 );
			node->SetCookie( path );

			InsertNode( node );			
			printf( "%s\n", cName.c_str());
i++;
		}

	} catch(...) { }
}

void FolderTreeView :: ReadRoot()
{
	Volumes cVolumes;
	String cMountPoint;

	for( uint i = 0; i < cVolumes.GetMountPointCount(); i++ )
	{
		if( cVolumes.GetMountPoint( i, &cMountPoint ) == 0 )
		{
			fs_info sInfo;
			
			if( cVolumes.GetFSInfo( i, &sInfo ) == 0 && ( sInfo.fi_flags & FS_IS_BLOCKBASED ) )
			{
				String zVolumeName = sInfo.fi_volume_name;
				if( ( zVolumeName == "" ) || ( zVolumeName == "/" ) )
					zVolumeName = "Unknown";				
			
				// Add drive to tree
				TreeViewStringNode* node = new TreeViewStringNode();
				node->AppendString( zVolumeName );
				node->SetIndent( 1 );
				node->SetCookie( cMountPoint );	
				InsertNode( node );		

				// Check if there is subdirectories
				if( FindSubDir( cMountPoint.c_str() ) )
				{
					// Insert a empty item
					TreeViewStringNode* node = new TreeViewStringNode();
					node->AppendString( "" );
					node->SetIndent( 2 );
					InsertNode( node );		
					TreeViewNode* parent = GetParent( node );
					parent->SetExpanded( false );
				}

			}
		}
	}
}

bool FolderTreeView :: FindSubDir( String path )
{
	// Check if there is any subdirectories
	try
	{
		String cName;
		Directory dir( path );
		dir.Rewind();

		while( dir.GetNextEntry( &cName ) == 1 )
		{
			if( cName != "." && cName != ".." )
			{
				printf( "  %s\n", cName.c_str() );
				// Check if this is a folder
				FSNode fs( path + '/' + cName );				
				if( fs.IsDir() )
					return true;
			}
		}
	}
	catch( ... ) { }

	return false;
}

void FolderTreeView :: DisplayPath( int row, TreeViewNode* parent, String path )
{
	list<String> list_of_dirs;

printf( "%s\n", path.c_str() );

	try
	{
		String cName;
		Directory dir( path );
		dir.Rewind();

		while( dir.GetNextEntry( &cName ) == 1 )
		{
			if( cName != "." && cName != ".." )
			{
				printf( "  %s\n", cName.c_str() );
				// Check if this is a folder
				FSNode fs( path + '/' + cName );				
				if( fs.IsDir() )
					list_of_dirs.push_back( cName );
			}
		}
	}
	catch( ... ) { }

	// sort the list
	list_of_dirs.sort();

	// reverse the list
	list_of_dirs.reverse();

	for( list<String>::iterator it = list_of_dirs.begin() ; it != list_of_dirs.end() ; it++ )
	{
		// Add drive to tree
		TreeViewStringNode* node = new TreeViewStringNode();
		node->AppendString( *it );
		node->SetIndent( parent->GetIndent() + 1 );
		node->SetCookie( path );	
		InsertNode( row + 1, node );

		// Check if there is subdir
		if( FindSubDir( path + '/' + *(it)) )
		{
			// Insert a empty item
			TreeViewStringNode* empty = new TreeViewStringNode();
			empty->AppendString( "" );
			empty->SetIndent( node->GetIndent() + 1 );
			InsertNode( row + 2, empty );					
		}
	}
}


void FolderTreeView :: ExpandItem( int row, TreeViewNode* parent )
{
	// Remove all all children
	TreeViewNode* child = GetChild( parent );

	while( child != NULL )
	{
		printf( "Deleting child\n" );
		RemoveNode( row + 1 );
		child = GetChild( parent );
	}	

	DisplayPath( row, parent, parent->GetCookie().AsString() );
}

void FolderTreeView :: CollapseItem( int row, TreeViewNode* parent )
{
	// Remove all items but create an empty
	TreeViewNode* child = GetChild( parent );

	while( child != NULL )
	{
		RemoveNode( row + 1 );
		child = GetChild( parent );
	}	

	// Insert a empty item
	TreeViewStringNode* empty = new TreeViewStringNode();
	empty->AppendString( "" );
	empty->SetIndent( parent->GetIndent() + 1 );
	InsertNode( empty );		
}
